# frozen_string_literal: true
# checker Api::Dmo::HermesController avant d'ajouter des méthodes
module Hermes
  module Api
    module Dmo
      class OperationsController < Hermes::Api::Dmo::ApplicationController

        VALID_OPERATORS = %w[
          $in
          $nin
          $ne
          $gt
          $gte
          $lt
          $lte
          $regex
        ].freeze

        ACCEPTABLE_PARAMS = %w[
          name
          code
          lessor_id
        ].freeze

        REGEX_PARAMS = %w[
        ].freeze

        MUTATE_PARAMS = %i[
          name
          code
          lessor_id
        ].freeze
        mutate_params_only(*MUTATE_PARAMS)

        def set_instance
          @instance = nil
          lessor_ids = Hermes::Lessor.readable_by(@user).pluck(:id).map(&:to_s)
          if params[:id]
            @instance = @model.where({
              id: params[:id],
              '$or': [
                {lessor_id: { '$in': lessor_ids}},
                {lessor_id: nil},
              ]
            }).first
          end
        end

        def is_admin
          (
            @user.auth_roles&.include?('superadministrateur') || 
            @user.roles&.include?('Administrator') || 
            @user.roles&.include?('SuperAdmin') || 
            @user.roles&.include?('Manager')
          )
        end

        def create
          if !is_admin && (!mutate_valid_params[:lessor_id] || !Hermes::Lessor.find_by(id: mutate_valid_params[:lessor_id]).readable_by?(@user))
            return render_json_error(:unprocessable_entity, :cannot_create_without_lessor_id)
          end
          super
        end

        def user_can_destroy?
          if is_admin || (
            @instance.lessor_id && 
            Hermes::Lessor.find_by(id: @instance.lessor_id).readable_by?(@user)
          )
            return true
          end
          false
        end

        def index
          options = {}

          if params[:lessor_id]
            operations = @model.where({lessor_id: params[:lessor_id]})
          else
            lessor_ids = Hermes::Lessor.all().readable_by(@user).map { |l| l.id.to_s}
            operations =  @model.where({
              '$or': [
                {lessor_id: { '$in': lessor_ids}},
                {lessor_id: nil},
              ] 
            })
          end

          paginate(
            operations,
            each_serializer: @serializer,
            **@options
          )
        end
      end
    end
  end
end
